# $Id: ISA.tcl,v 1.16 2002/07/05 12:43:29 rrp Exp $	
package require vtk
package require vtkinteraction

source ISA_misc.tcl
source ISA_filehandling.tcl
source LMPoint.tcl

# Global variables
########################
set maintitle "IMM Surface Annotation for VTK 4.x v.0.3.1"

# The numbers on the landmarks
set viewNumbers 0
# View the plane that defines the cut
set viewCutPlane 0
# View the clipped part
set viewClip 1
# View the cutted outline
set viewCut 1
# View the axes
set viewAxes 1
# View original data
set viewOrgData 1

# Create GUI and top some menu
########################
#wm withdraw .
#toplevel .top
wm title . $maintitle

frame .mbar -relief raised -bd 2
pack .mbar -side top -fill x

menubutton .mbar.file -text File -menu .mbar.file.menu
menubutton .mbar.edit -text Edit -menu .mbar.edit.menu
menubutton .mbar.view -text View -menu .mbar.view.menu
pack .mbar.file .mbar.edit .mbar.view -side left

menu .mbar.file.menu
   .mbar.file.menu add command -label "Open file" -command {OpenFile {}}
   .mbar.file.menu add command -label "Load Landmark Points" -command {OpenLMPointFile {}}
   .mbar.file.menu add command -label "Save Landmark Points" -command {SaveLMPointFile}
   .mbar.file.menu add command -label "Read Plane" -command {OpenPlaneFile {}}
   .mbar.file.menu add command -label "Save Plane" -command {SavePlane}
   .mbar.file.menu add command -label "Save cut profile" -command {SaveCutProfile}
   .mbar.file.menu add command -label Exit -command ExitConfirmation
menu .mbar.edit.menu
   .mbar.edit.menu add command -label "Clear landmarks" -command {LMP_ClearPoints}
menu .mbar.view.menu
   .mbar.view.menu add check -label "View numbers" -variable viewNumbers -command {LMP_ViewNumbers}
   .mbar.view.menu add check -label "View original data" -variable viewOrgData -command {ViewOrgDataOnOff}
   .mbar.view.menu add check -label "View plane" -variable viewCutPlane -command {ViewCutPlaneOnOff}
   .mbar.view.menu add check -label "View clipped part" -variable viewClip -command {ViewClipOnOff}
   .mbar.view.menu add check -label "View cutted outline" -variable viewCut -command {ViewCutOnOff}
   .mbar.view.menu add check -label "View axes" -variable viewAxes -command {ViewAxesOnOff}

frame .topf -relief sunken -borderwidth 2
checkbutton .topf.viewnumbers -text "Numbers" -variable viewNumbers -command {LMP_ViewNumbers}
checkbutton .topf.vieworg -text "Org (o)" -variable viewOrgData -command {ViewOrgDataOnOff}
checkbutton .topf.viewplane -text "Plane (b)" -variable viewCutPlane -command {ViewCutPlaneOnOff}
checkbutton .topf.viewclip -text "Clipped (v)" -variable viewClip -command {ViewClipOnOff}
checkbutton .topf.viewcut -text "Outline" -variable viewCut -command {ViewCutOnOff}
checkbutton .topf.viewaxes -text "Axes" -variable viewAxes -command {ViewAxesOnOff}
button .topf.doclip -text "Clip! (n)" -command {ClipPolysWithPlane}

pack .topf.viewnumbers .topf.vieworg .topf.viewplane .topf.viewclip .topf.viewcut .topf.viewaxes .topf.doclip -side left -anchor w -fill x -padx 0 -pady 0
pack .topf -side top -anchor w -fill x -padx 0 -pady 0


frame .f1
vtkTkRenderWidget .f1.r1 -width 600 -height 400
pack .f1.r1 -side left -padx 3 -pady 3 -fill both -expand t
pack .f1 -fill both -expand t

set PointList [Scrolled_Listbox .f1.pointlist -width 40 -height 30]
pack .f1.pointlist -side right -fill y

# Status bar
frame .bottomF -relief sunken -borderwidth 3
label .bottomF.status -text "(No data)" -borderwidth 0
label .bottomF.currentP -text "(No current point)" -borderwidth 0
label .bottomF.polyInfo -text "(No date)" -borderwidth 0
#pack .bottomF.status -side top -anchor w -fill x -padx 0 -pady 0
pack .bottomF.polyInfo .bottomF.currentP .bottomF.status -side right -anchor e -fill x -padx 0 -pady 0
pack .bottomF  -side top -anchor w -fill x -padx 0 -pady 0

#BindTkRenderWidget .f1.r1
::vtk::bind_tk_render_widget .f1.r1
RRP_BindTkRenderWidget .f1.r1
RRP_BindTkRenderWidget PointList

# Initialise point structures
LMP_Init


# Initialise VTK objects
########################
set renWin [.f1.r1 GetRenderWindow]
vtkRenderer ren1
$renWin AddRenderer ren1

set CurrentRenderer ren1
set RendererFound 1

# Rasmus test
set iren [$renWin GetInteractor]
set istyle [vtkInteractorStyleTrackballCamera istyletrackcam]
$iren SetInteractorStyle $istyle

# Using this will cause the a key to switch to actor mode 
# thus creating a conflict with the annotatepoint call
#set istyle [vtkInteractorStyleSwitch istyleswitch]
#$iren SetInteractorStyle $istyle
#$istyle SetCurrentStyleToTrackballCamera

#$iren AddObserver CharEvent CharCallback

vtkPolyData PolyData

vtkPolyDataMapper Mapper
    Mapper SetInput PolyData
vtkProperty property
    property SetColor 0.2400 0.3500 0.6700
    property SetSpecularColor 1 1 1
    property SetSpecular 0.3
    property SetSpecularPower 20
    property SetAmbient 0.2
    property SetDiffuse 0.8
    property SetOpacity 1.0
vtkActor Actor
   Actor SetMapper Mapper
   Actor SetProperty property

# Visualise axes
vtkAxes axes
    axes SetOrigin 0 0 0
    axes SetScaleFactor 1
vtkPolyDataMapper axesMapper
    axesMapper SetInput [axes GetOutput]
vtkActor axesActor
    axesActor SetMapper axesMapper
    axesActor PickableOff

ren1 AddActor Actor
ren1 AddActor axesActor

ren1 SetBackground 0.8 0.8 0.8

#Make Spere
vtkSphereSource pickerSphere
   pickerSphere SetThetaResolution 8
   pickerSphere SetPhiResolution 8
   pickerSphere SetRadius 0.35
vtkPolyDataMapper pickerSphereMapper;
   pickerSphereMapper  SetInput [pickerSphere GetOutput]
vtkActor pickerSphereActor
   pickerSphereActor PickableOff; #We are not able to pick the Sphere itself
   eval [pickerSphereActor GetProperty] SetColor 0 1 0
   pickerSphereActor SetMapper pickerSphereMapper


# Make a plane widget
vtkPlaneWidget planeWidget
planeWidget SetResolution 20
planeWidget SetRepresentationToSurface
planeWidget SetInteractor $iren
[planeWidget GetPlaneProperty] SetOpacity 0.8
#[planeWidget GetPlaneProperty] SetColor 0.55 0.9 0.9
[planeWidget GetPlaneProperty] SetColor 0.0 0.8 0.8

# make a clipped output poly
vtkPolyData ClippedPart

vtkPolyDataMapper ClippedMapper
    ClippedMapper SetInput ClippedPart
vtkProperty ClipProp
    ClipProp SetColor 0.0 1.0 0.0
    ClipProp SetSpecularColor 1 1 1
    ClipProp SetSpecular 0.3
    ClipProp SetSpecularPower 20
    ClipProp SetAmbient 0.2
    ClipProp SetDiffuse 0.8
    ClipProp SetOpacity 1.0
vtkActor ClippedActor
   ClippedActor SetMapper ClippedMapper
   ClippedActor SetProperty ClipProp

ren1 AddActor ClippedActor

# make a cutted output poly
# the cutted output is the cut outline
vtkPolyData CutPart

vtkPolyDataMapper CutMapper
    CutMapper SetInput CutPart
vtkProperty CutProp
    CutProp SetColor 0 0 0
    CutProp SetSpecularColor 0 0 0
    CutProp SetSpecular 0.0
    CutProp SetSpecularPower 0
    CutProp SetAmbient 0.0
    CutProp SetDiffuse 0.0
    CutProp SetOpacity 1.0
    CutProp SetLineWidth 3

vtkActor CutActor
   CutActor SetMapper CutMapper
   CutActor SetProperty CutProp

ren1 AddActor CutActor

vtkPointPicker picker
    picker SetTolerance 0.01; #default 1/40 = 0.025
proc AnnotatePoint {widget x y} {
    global CurrentRenderer RendererFound
 
    puts "Annotate new point"

    if { ! $RendererFound } { return }

    set WindowY [lindex [$widget configure -height] 4]
    picker Pick $x [expr $WindowY - $y - 1] 0.0 $CurrentRenderer

    puts stdout "pointId = [picker GetPointId]"
    puts stdout "tolerance = [picker GetTolerance]"
    set ptId [picker GetPointId]
    
    if {$ptId >= 0} {
	puts stdout "--------------------------------------"
	puts stdout "Picked Point id = $ptId"
	puts stdout "Picked point (x y z) = [picker GetPickPosition]"
	
	LMP_pushPoint $ptId [picker GetPickPosition]
	
	StatusMessage "Picked point (x y z) = ([picker GetPickPosition])"
    } else {
	puts "No point picked"
	StatusMessage "No point picked"
    }    

    $widget Render
}

proc InsertPoint {widget x y} {
    global CurrentRenderer RendererFound
 
    puts "Inserting/moving point"

    if { ! $RendererFound } { return }

    set WindowY [lindex [$widget configure -height] 4]
    picker Pick $x [expr $WindowY - $y - 1] 0.0 $CurrentRenderer
    
    puts stdout "pointId = [picker GetPointId]"
    puts stdout "tolerance = [picker GetTolerance]"
    set ptId [picker GetPointId]
    
    if {$ptId >= 0} {
	puts stdout "--------------------------------------"
	puts stdout "Picked Point id = $ptId"
	puts stdout "Picked point (x y z) = [picker GetPickPosition]"
	
	LMP_setTheCurrentPoint $ptId [picker GetPickPosition]

      StatusMessage "Picked point (x y z) = ([picker GetPickPosition])"
       } else {
	   StatusMessage "No point picked"
       }    

    $widget Render
}

bind $PointList <ButtonRelease-1> {UpdatePointList}

proc UpdatePointList {} {
   global PointList
   LMP_SetCurrentPoint [$PointList curselection]
}


# 
# Prints status message on message line
#
proc StatusMessage {s} {
    .bottomF.status configure -text $s
}

proc ShowCurrentPoint {s} {
    .bottomF.currentP configure -text "(Current Point: $s)"
}


# Procedure updates data statistics and GUI menus
#
proc UpdateGUI {} {
    global LMPoints
    
    set NumberOfNodes [PolyData GetNumberOfPoints]
    set NumberOfElements [PolyData GetNumberOfCells]
    
    set s [format "(Vertices:%d    Cells:%d)" \
	       $NumberOfNodes $NumberOfElements]
    
    .bottomF.polyInfo configure -text $s

    planeWidget SetInput PolyData
    planeWidget SetPlaceFactor 1.00
    planeWidget PlaceWidget

    ren1 ResetCamera
}

# Procedure - ExitConfirmation
proc ExitConfirmation {} {
    if {[tk_messageBox -icon question -default no \
	     -title "Exit" -message "Do you really want to quit?" -type yesno]=="yes"} {
	::vtk::cb_exit
	exit
    }
}

proc ViewCutPlaneOnOff {} {
    global viewCutPlane
    
    if {$viewCutPlane == 1} {
	planeWidget On 
    } else {
	planeWidget Off
    }
}

proc ViewCutPlaneToggle {} {
    global viewCutPlane
    
    if {$viewCutPlane == 1} {
	set viewCutPlane 0
    } else {
	set viewCutPlane 1
    }
    ViewCutPlaneOnOff
}

proc ViewOrgDataOnOff {} {
    global viewOrgData
    
    if {$viewOrgData == 1} {
	Actor VisibilityOn
    } else {
	Actor VisibilityOff
    }
    .f1.r1 Render
}

proc ViewOrgDataToggle {} {
    global viewOrgData
    
    if {$viewOrgData == 1} {
	set viewOrgData 0
    } else {
	set viewOrgData 1
    }
    ViewOrgDataOnOff
}

proc ViewClipOnOff {} {
    global viewClip
    
    if {$viewClip == 1} {
#	Actor VisibilityOff
	ClippedActor VisibilityOn
    } else {
#	Actor VisibilityOn
	ClippedActor VisibilityOff
    }
    .f1.r1 Render
}

proc ViewCutOnOff {} {
    global viewCut
    
    if {$viewCut == 1} {
	CutActor VisibilityOn
    } else {
	CutActor VisibilityOff
    }
    .f1.r1 Render
}

proc ViewAxesOnOff {} {
    global viewAxes
    
    if {$viewAxes == 1} {
	axesActor VisibilityOn
    } else {
	axesActor VisibilityOff
    }
    .f1.r1 Render
}

proc ViewClipToggle {} {
    global viewClip
    
    if {$viewClip == 1} {
	set viewClip 0
    } else {
	set viewClip 1
    }
    ViewClipOnOff
}

proc ViewCutToggle {} {
    global viewCut
    
    if {$viewCut == 1} {
	set viewCut 0
    } else {
	set viewCut 1
    }
    ViewCutOnOff
}


proc ClipPolysWithPlane {} {
    global PolyData
    global viewClip
    
    vtkPlane cpplane
    set porigin [planeWidget GetOrigin]
    set pnormal [planeWidget GetNormal]
    #    puts "Clip plane: ($porigin) ($pnormal)"
    cpplane SetOrigin [lindex $porigin 0] [lindex $porigin 1] [lindex $porigin 2]
    cpplane SetNormal [lindex $pnormal 0] [lindex $pnormal 1] [lindex $pnormal 2]
    
    vtkClipPolyData clipper
    clipper SetInput PolyData
    clipper SetClipFunction cpplane
    clipper GenerateClipScalarsOff
    clipper GenerateClippedOutputOn
    clipper SetValue 0.0
    clipper InsideOutOn
    
    vtkPolyDataConnectivityFilter connect
    connect SetInput [clipper GetOutput]
    connect SetExtractionModeToLargestRegion 
    connect Update
    
    ClippedPart CopyStructure [connect GetOutput]
    [ClippedPart GetPointData] PassData [[connect GetOutput] GetPointData]
    ClippedPart Modified
    
    vtkCutter cutter
    cutter SetInput PolyData
    cutter SetCutFunction cpplane
    cutter GenerateCutScalarsOff

    vtkPolyDataConnectivityFilter connect2
    connect2 SetInput [cutter GetOutput]
    connect2 SetExtractionModeToLargestRegion 
    connect2 Update
    
    CutPart CopyStructure [connect2 GetOutput]
    [CutPart GetPointData] PassData [[connect2 GetOutput] GetPointData]
    CutPart Modified
    
    cutter Delete
    clipper Delete
    cpplane Delete
    connect Delete
    connect2 Delete
    
#    set viewClip 1
#    set viewOrgData 0
#    ViewClipOnOff
#    viewOrgDataOnOff
    
    .f1.r1 Render
}


# Not used at the moment
#proc CharCallback {} {
#    global iren
#
#    puts "CharCallback"
#    set keycode [$iren GetKeyCode]
#    puts "Keycode $keycode"
    
#     if {$keycode != "n" && $keycode != "N"} {
#         return
#     }
    
#     set pos [$iren GetEventPosition]
#     puts "Pos: $pos"
#    ButtonCallback [lindex $pos 0] [lindex $pos 1]
#}


#OpenFile "D:\\data\\test\\BF9931237_pc_simp_orient.vtk"

set logfile $env(TEMP)\\isa_log.txt

puts "Errors and log messages goes to $logfile"

# Avoid silly and useless error windows and drop messages to a file instead
vtkFileOutputWindow outwin
#outwin SetFileName "C:/temp/isa_log.txt"
outwin SetFileName $logfile
outwin SetInstance outwin

StatusMessage "Log file: $logfile  "
wm protocol . WM_DELETE_WINDOW ::vtk::cb_exit

