# $Id: LMPoint.tcl,v 1.3 2002/07/01 14:30:03 rrp Exp $	
# This is an encapsulation of a set of landmark points (LMPoints)
# id is used as the index variable
# ex. the index of the position of point number 17 is pos,17

proc LMP_Init {} {
   global LMPoints
   set LMPoints(currentPoint) 0
   set LMPoints(MaxPoint) 0
}

proc LMP_UpdateDisplayOfPoint {id} {
   global LMPoints
   global PointList

   set pos $LMPoints(pos,$id)
   set isSet $LMPoints(isSet,$id)

   set text "$id : ([lindex $pos 0], [lindex $pos 1], [lindex $pos 2])"

   if {$isSet == 0} {
      set text "$id : -" 
   }

   $PointList delete $id
   $PointList insert $id $text

   LMP_UpdateDisplayOfCurrentPoint
}

proc LMP_setPoint {id ptID pos} {
   global LMPoints
   global CurrentRenderer
   global viewNumbers
 
   if {$id >= $LMPoints(MaxPoint)} {return}

   # delete old actor
   if {$LMPoints(isSet,$id) == 1} {
      LMP_unsetPoint $id
   }

   set LMPoints(ptID,$id) $ptID
   set LMPoints(pos,$id)  $pos
   set LMPoints(isSet,$id) 1

   pickerSphereActor$id SetPosition [lindex $pos 0] [lindex $pos 1] [lindex $pos 2]
   textActor$id SetPosition [lindex $pos 0] [lindex $pos 1] [lindex $pos 2]

#   $CurrentRenderer AddActor pickerSphereActor$id
   
   pickerSphereActor$id VisibilityOn
   if {$viewNumbers == 1} {
       textActor$id VisibilityOn
   }

   LMP_UpdateDisplayOfPoint $id
}

proc LMP_setTheCurrentPoint {ptID pos} {
   global LMPoints

   set id $LMPoints(currentPoint)
   LMP_setPoint $id $ptID $pos
}

# The only way to add points
proc LMP_pushPoint {ptID pos} {
   global LMPoints
   global viewNumbers
   global CurrentRenderer

   set id $LMPoints(MaxPoint)
   incr LMPoints(MaxPoint)

   # Here the VTK classes for this point are created (It is checked if they exists)
   if {[info commands pickerSphereActor$id] == ""} {
   vtkActor pickerSphereActor$id }
    pickerSphereActor$id PickableOff; #We are not able to pick the Sphere itself
    eval [pickerSphereActor$id GetProperty] SetColor 0 1 0
    pickerSphereActor$id SetMapper pickerSphereMapper

   
# -------------------------- old text --------------
   if {[info commands atext$id] == ""} {
    vtkVectorText atext$id}
       atext$id SetText "$id"
   if {[info commands textMapper$id] == ""} {
    vtkPolyDataMapper textMapper$id}
       textMapper$id SetInput [atext$id GetOutput]
   if {[info commands textActor$id] == ""} {
	vtkFollower textActor$id}
      textActor$id SetMapper textMapper$id
	  textActor$id SetScale 1.0 1.0 1.0
	  textActor$id SetPosition [lindex $pos 0] [lindex $pos 1] [lindex $pos 2]
      [textActor$id GetProperty] SetColor 1.0 1.0 1.0
      textActor$id SetCamera [$CurrentRenderer GetActiveCamera]


# -------------------------- test of text --------------
#vtkTextMapper textMapper$id
#    textMapper$id SetInput $id
#    textMapper$id SetFontSize 18
#    textMapper$id SetFontFamilyToArial
#    textMapper$id SetJustificationToCentered
#    textMapper$id BoldOn
#    textMapper$id ItalicOn
#    textMapper$id ShadowOn
#vtkScaledTextActor textActor$id
#    textActor$id SetMapper textMapper$id
#    [textActor$id GetProperty] SetColor 0 0 1

 #   set coord [textActor$id GetPositionCoordinate]
#	$coord SetCoordinateSystemToWorld
#	$coord SetValue [lindex $pos 0] [lindex $pos 1] [lindex $pos 2]
# -------------------------------

   set LMPoints(ptID,$id) $ptID
   set LMPoints(pos,$id)  $pos
   set LMPoints(isSet,$id) 1

   pickerSphereActor$id SetPosition [lindex $pos 0] [lindex $pos 1] [lindex $pos 2]
   pickerSphereActor$id VisibilityOn

   if {$viewNumbers == 0} {
      textActor$id VisibilityOff
   } else {
      textActor$id VisibilityOn
   }

   $CurrentRenderer AddActor pickerSphereActor$id
   $CurrentRenderer AddActor textActor$id

   LMP_SetCurrentPoint $id

   LMP_UpdateDisplayOfPoint $id
}

proc LMP_ClearPoints {} {
   global PointList
   global LMPoints
   global CurrentRenderer

   for {set id 0} {$id < $LMPoints(MaxPoint)} {incr id} {
 
#      $CurrentRenderer RemoveActor pickerSphereActor$id
#      $CurrentRenderer RemoveActor textActor$id
      
#	  pickerSphereActor$id Delete
#	  textActor$id Delete

       textActor$id VisibilityOff
	   pickerSphereActor$id VisibilityOff

#      unset LMPoints(pos,$id)
#      unset LMPoints(isSet,$id)
#      unset LMPoints(ptID,$id)
#   }
   set LMPoints(currentPoint) 0
   set LMPoints(MaxPoint) 0

   # Clear listbox
   $PointList delete 0 end

   StatusMessage "Cleared points"

   LMP_UpdateDisplayOfCurrentPoint

#   Render .f1.r1
   .f1.r1 Render
}

proc LMP_unsetPoint {id} {
   global LMPoints
   global CurrentRenderer

   if {$LMPoints(isSet,$id) == 1} {
      set LMPoints(isSet,$id) 0

#      $CurrentRenderer RemoveActor pickerSphereActor$id
	  pickerSphereActor$id VisibilityOff
      textActor$id VisibilityOff

      LMP_UpdateDisplayOfPoint $id

#      Render .f1.r1    

     .f1.r1 Render
   }
}

proc LMP_UnsetCurrentPoint {} {
   global LMPoints

   set id $LMPoints(currentPoint)
   LMP_unsetPoint $id
}

proc LMP_ptID {id} {
   global LMPoints
   return $LMPoints(ptID,$id)
}

proc LMP_pos {id} {
   global LMPoints
   return $LMPoints(pos,$id)
}

proc LMP_printPoints {} {
   global LMPoints

   puts "Maxpoint: $LMPoints(MaxPoint)"
   puts "CurrentPoints: $LMPoints(currentPoint)"

   for {set id 0} {$id < $LMPoints(MaxPoint)} {incr id} {
      set pos $LMPoints(pos,$id)
      set isSet $LMPoints(isSet,$id)
      
	  set text "$id : ([lindex $pos 0], [lindex $pos 1], [lindex $pos 2])"

      if {$isSet == 0} {
         set text "$id : -" 
         }
      puts $text
   }
}

proc LMP_makePointCloud {} {
   LMP_SetCurrentPoint 5
}

proc LMP_UpdateDisplayOfCurrentPoint {} {
   global LMPoints
   global PointList

   set current $LMPoints(currentPoint) 
   ShowCurrentPoint $current

   if {$LMPoints(MaxPoint) == 0} {return}

   $PointList selection clear 0 end
   $PointList selection set $current

   # set all to green. Very uneffective
   for {set i 0} {$i < $LMPoints(MaxPoint)} {incr i} {
       eval [pickerSphereActor$i GetProperty] SetColor 0 1 0
       }

   eval [pickerSphereActor$current GetProperty] SetColor 1 0 0

#    puts "Update a"

    set cp $LMPoints(currentPoint)
    set p1 [expr $cp-1]
    set p2 [expr $cp-2]
    if {$cp  >= 2} {
#       puts "Update b"
        set pos1 $LMPoints(pos,$cp)
        set pos2 $LMPoints(pos,$p1)
        set pos3 $LMPoints(pos,$p2)

#	puts "pos1 : ($pos1) pos2 : ($pos2) pos3 : ($pos3)"

#	cutPlane SetOrigin [lindex $pos1 0] [lindex $pos1 1] [lindex $pos1 2]
#	cutPlane SetPoint1 [lindex $pos2 0] [lindex $pos2 1] [lindex $pos2 2]
#	cutPlane SetPoint2 [lindex $pos3 0] [lindex $pos3 1] [lindex $pos3 2]
    }

#   Render .f1.r1
   .f1.r1 Render
}

proc LMP_SetCurrentPoint {current} {
   global LMPoints
   global PointList

   set LMPoints(currentPoint) $current

   LMP_UpdateDisplayOfCurrentPoint
}

proc LMP_IncCurrentPoint {} {
   global LMPoints

   incr LMPoints(currentPoint)

   if {$LMPoints(currentPoint) >= $LMPoints(MaxPoint)} {set LMPoints(currentPoint) 0}
  
   LMP_UpdateDisplayOfCurrentPoint
}

proc LMP_DecCurrentPoint {} {
   global LMPoints
 
   if {$LMPoints(MaxPoint) > 0} {
      incr LMPoints(currentPoint) -1

      set mt [expr $LMPoints(MaxPoint)-1]

      if {$LMPoints(currentPoint) < 0} {set LMPoints(currentPoint) $mt}
  
      LMP_UpdateDisplayOfCurrentPoint
	}
}

proc LMP_ViewNumbers {} {
    global viewNumbers
	global LMPoints
	global CurrentRenderer

    if {$LMPoints(MaxPoint) == 0} { return}

   for {set id 0} {$id < $LMPoints(MaxPoint)} {incr id} {
       if {$viewNumbers == 1 && $LMPoints(isSet,$id) == 1} {
           textActor$id VisibilityOn
	   } else {
           textActor$id VisibilityOff
       }
    }

#    StatusMessage "ViewNumbers: $viewNumbers"

#    Render .f1.r1
    .f1.r1 Render
}

proc LMP_WritePoints { fileID } {
   global LMPoints

   puts $fileID $LMPoints(MaxPoint)

   for {set id 0} {$id < $LMPoints(MaxPoint)} {incr id} {
      set pos $LMPoints(pos,$id)
      set isSet $LMPoints(isSet,$id)
	  set ptID  $LMPoints(ptID,$id)
      
	  set text "1 $ptID $pos"

      if {$isSet == 0} {
         set text "0 0 0 0 0" 
         }
      puts $fileID $text
   }
}

proc LMP_WritePointsInVTKFormat { fileName } {
    global LMPoints
    
    vtkPoints pts
    vtkCellArray verts
    vtkPolyData pd
    vtkFloatArray scalars
    scalars SetNumberOfComponents 1
    
    for {set id 0} {$id < $LMPoints(MaxPoint)} {incr id} {
	set pos $LMPoints(pos,$id)
	set isSet $LMPoints(isSet,$id)
	set ptID  $LMPoints(ptID,$id)

	if {$isSet == 0} {
	    set pos "0 0 0" 
	}
	set tid [pts InsertNextPoint [lindex $pos 0] [lindex $pos 1] [lindex $pos 2]]
	verts InsertNextCell 1
	verts InsertCellPoint $tid
	scalars InsertNextValue $isSet
    }
    pd SetPoints pts
    pd SetVerts verts
    [pd GetPointData] SetScalars scalars
    vtkPolyDataWriter writer
     writer SetInput pd
     writer SetFileName $fileName
     writer Write

    writer Delete
    verts Delete
    scalars Delete
    pts Delete
    pd Delete
}

proc LMP_ReadPoints { fileID } {
    global LMPoints
    
    LMP_ClearPoints
    
    set max [gets $fileID]
    
    for {set id 0} {$id < $max} {incr id} {
	set temp [gets $fileID]
	
	set isSet [lindex $temp 0]
	set ptID  [lindex $temp 1]
	set pos   "[lindex $temp 2] [lindex $temp 3] [lindex $temp 4]"
	
	LMP_pushPoint $ptID $pos
	
	if {$isSet == 0} {
	    LMP_unsetPoint $id
	}
    }
}

proc LMP_ReadPointsInVTKFormat { filename } {
    global LMPoints
    
    # Security delete
    if { [info commands reader] != "" } {reader Delete}
    
    vtkPolyDataReader reader
    reader SetFileName $filename
    reader Update
    
    LMP_ClearPoints
    
    set max [[reader GetOutput] GetNumberOfPoints]
    
    puts "number of points $max" 
    
    for {set id 0} {$id < $max} {incr id} {
	set ptID $id
	set pos   [[reader GetOutput] GetPoint $id]
	set isSet 1

	if {[[[reader GetOutput] GetPointData] GetScalars] != ""} {
	    set isSet [[[[reader GetOutput] GetPointData] GetScalars] GetValue $id]	    
	}
	
	LMP_pushPoint $ptID $pos
	
	if {$isSet == 0} {
	    LMP_unsetPoint $id
	}
    }
    reader Delete
}
